/****************************************************************************/
/*  Test DMX512                                                             */
/****************************************************************************/
/*  LEnsE / Julien VILLEMEJANE     /     Rémi GAROT--JACQUEY / Elisa JARRY  */
/****************************************************************************/
/*  Brochage                                                                */
/*  |   Entrée son : A2                                                     */
/*  |   Joystick X : D12                                                    */
/*  |   Joystick Y : D13                                                    */
/****************************************************************************/
/*  Test réalisé sur Nucléo-L476RG                                          */
/****************************************************************************/

#include "mbed.h"
#include "platform/mbed_thread.h"
#include <cstdlib>

#define SAMPLES  512 
#define ACQ_TIME 1
#define ACQ_FREQ 25000 //fréquence d'échantillonage
#define TAILLE_ECHANTILLON 64 //Nombre qu'on prend dans un échantillon
#define PRECISION 2 //Paramètre du seuil pour détecter le dépassement

Serial      debug_pc(USBTX, USBRX);
Serial      dmx(PA_0, PA_1);
DigitalOut  out_tx(D5);
DigitalOut  start(D4);     //envoie des données
DigitalOut  enableDMX(D6);
AnalogIn    x1_axis(D12); //joystick
AnalogIn    y1_axis(D13);
//BPM
AnalogIn    mic(A2);

Ticker mesure;
Ticker update_dmx;

// DMX
char dmx_data[SAMPLES] = {0}; char nb = 0;
void initDMX(); void updateDMX();

// Définition BPM
void ticmesure(void);
void ticUpdateDMX(void); void ticUpdateDMX2(void);

int echantillons_mesures=0; int mesure_dans_echantillon=0;
float somme_mesures_echantillon=0.0;
float temp;
float energie_moyenne=100.0; float energie_et=0.0; //moyenne et écart-type
float calcul_mean=0.0; float calcul_std=100.0; //variable mesure somme pour moyenne et écart-type
bool isAlasuite=false; bool isAlasuite2=false; //savoir si on a déjà récemment changé la lumière
int upordowndmx[4]={0,1,1,1}; //savoir si on doit monter ou descendre (si on est proche du seuil)

int main(){
    debug_pc.baud(9600); debug_pc.printf("Lancement programme BPM / Musique en Rythme\n");
    initDMX();
    dmx_data[0]=255;dmx_data[1]=255; dmx_data[2]=0;dmx_data[3]=0; //RGB LED
    dmx_data[45] = 255; dmx_data[46] = 0; dmx_data[47] = 0; //RGB Lyre
    dmx_data[42] = 100; //Speed lyre
    dmx_data[43] = 100; //Dimmer lyre
    updateDMX();

    srand(time(NULL));
    mesure.attach(&ticmesure,0.000040); //on fait les mesures tous les 40us
    while(1);
}



void ticmesure(void){
        dmx_data[40] = x1_axis.read()*255; //PAN movement pas dans main car sinon trop rapide pour le chgt updatedmx
        dmx_data[41] = y1_axis.read()*255;
        
        if (echantillons_mesures<=(ACQ_TIME * ACQ_FREQ)/TAILLE_ECHANTILLON){
            temp=mic.read()-0.5;
            temp=temp*temp;

            if (mesure_dans_echantillon<=TAILLE_ECHANTILLON){ //mesure de l'intensité sonore sur un échantillon
                somme_mesures_echantillon+=temp;
                mesure_dans_echantillon+=1;
            }

            else{//On compare la mesure d'un échantillon à une moyenne sur N échantillons
                if(somme_mesures_echantillon>energie_moyenne + 2 * energie_et){ //Si on détecte un grand changement
                    if(!isAlasuite){
                        ticUpdateDMX2(); //on fait le grand changement de lumière
                        isAlasuite=true;
                    }
                }
                else{ //on ne souhaite pas faire 2 changements à la suite, c'est trop proche.
                    isAlasuite=false;
                
                if(somme_mesures_echantillon>energie_moyenne * PRECISION){ //si on a un changement plus faible
                    if(!isAlasuite2){
                        ticUpdateDMX();//on fait un changement plus faible de la lumière
                        isAlasuite2=true;
                    }
                }
                else
                    isAlasuite2=false;
                }
                calcul_mean+=somme_mesures_echantillon;
                calcul_std+=somme_mesures_echantillon*somme_mesures_echantillon;
                somme_mesures_echantillon=temp;//on relance la nouvelle série de mesure
                mesure_dans_echantillon=1;
                echantillons_mesures+=1;
            }
        }

        else{
            energie_moyenne=calcul_mean/(1.0 * (ACQ_TIME * ACQ_FREQ) / TAILLE_ECHANTILLON);
            energie_et=pow( (calcul_std - energie_moyenne*energie_moyenne) / (1.0*(ACQ_TIME*ACQ_FREQ)/TAILLE_ECHANTILLON - 1) , 0.5);
            //On réinitialise les variables pour lancer une nouvelle acquisition
            calcul_mean=0.0;
            calcul_std=0.0;
            echantillons_mesures=0;
        } 
    
}

void ticUpdateDMX2(void){
    int i=1+rand()%3; // on choisit une couleur RGB à modifier
    if (dmx_data[i]>170) upordowndmx[i]=-1; //on fait en sorte de rester dans les bornes 0,255
    else if(dmx_data[i]<71) upordowndmx[i]=1;
    dmx_data[i]=dmx_data[i] + upordowndmx[i]*70;
    dmx_data[44+i]=dmx_data[i];
    updateDMX();
}

void ticUpdateDMX(void){
    //debug_pc.printf("update dmx\n");
    int i=1+rand()%3;// on choisit une couleur RGB à modifier
    if (dmx_data[i]>230) upordowndmx[i]=-1; //on fait en sorte de rester dans les bornes 0,255
    else if(dmx_data[i]<20) upordowndmx[i]=1;
    dmx_data[i]=dmx_data[i] + upordowndmx[i]*10;
    dmx_data[44+i]=dmx_data[i];
    updateDMX();
}


//les trucs de dmx
void initDMX(){
    // Initialisation DMX
    dmx.baud(250000);
    dmx.format (8, SerialBase::None, 2);
    enableDMX = 0;
    // Initialisation canaux DMX
    for(int k = 0; k < SAMPLES; k++){
        dmx_data[k] = 0;
    }    
    updateDMX();  
}

void updateDMX(){
        enableDMX = 1;
        start = 1;      // /start
        out_tx = 0;     // break
        wait_us(88);    
        out_tx = 1;     // mb
        wait_us(8);     
        out_tx = 0;     // break
        start = 0;
        dmx.putc(0);     // Start
        for(int i = 0; i < SAMPLES; i++){
            dmx.putc(dmx_data[i]);     // data
        }
        wait_us(23000);
}
